@@ -9,9 +9,10 @@ import enum
 import logging
 import os
 import sys
+from collections.abc import Iterator
 from contextlib import ExitStack, contextmanager
 from pathlib import Path
-from typing import IO, Iterator, NoReturn, cast
+from typing import IO, NoReturn, cast
 
 from pip_audit import __version__
 from pip_audit._audit import AuditOptions, Auditor
@@ -22,6 +23,7 @@ from pip_audit._dependency_source import (
     PyProjectSource,
     RequirementSource,
 )
+from pip_audit._dependency_source.pylock import PyLockSource
 from pip_audit._fix import ResolvedFixVersion, SkippedFixVersion, resolve_fix_versions
 from pip_audit._format import (
     ColumnsFormat,
@@ -30,9 +32,15 @@ from pip_audit._format import (
     MarkdownFormat,
     VulnerabilityFormat,
 )
-from pip_audit._service import OsvService, PyPIService, VulnerabilityService
+from pip_audit._service import EcosystemsService, OsvService, PyPIService
 from pip_audit._service.interface import ConnectionError as VulnServiceConnectionError
-from pip_audit._service.interface import ResolvedDependency, SkippedDependency
+from pip_audit._service.interface import (
+    Dependency,
+    ResolvedDependency,
+    SkippedDependency,
+    VulnerabilityResult,
+    VulnerabilityService,
+)
 from pip_audit._state import AuditSpinner, AuditState
 from pip_audit._util import assert_never
 
@@ -71,17 +79,17 @@ class OutputFormatChoice(str, enum.Enum):
     CycloneDxXml = "cyclonedx-xml"
     Markdown = "markdown"
 
-    def to_format(self, output_desc: bool) -> VulnerabilityFormat:
+    def to_format(self, output_desc: bool, output_aliases: bool) -> VulnerabilityFormat:
         if self is OutputFormatChoice.Columns:
-            return ColumnsFormat(output_desc)
+            return ColumnsFormat(output_desc, output_aliases)
         elif self is OutputFormatChoice.Json:
-            return JsonFormat(output_desc)
+            return JsonFormat(output_desc, output_aliases)
         elif self is OutputFormatChoice.CycloneDxJson:
             return CycloneDxFormat(inner_format=CycloneDxFormat.InnerFormat.Json)
         elif self is OutputFormatChoice.CycloneDxXml:
             return CycloneDxFormat(inner_format=CycloneDxFormat.InnerFormat.Xml)
         elif self is OutputFormatChoice.Markdown:
-            return MarkdownFormat(output_desc)
+            return MarkdownFormat(output_desc, output_aliases)
         else:
             assert_never(self)  # pragma: no cover
 
@@ -97,14 +105,7 @@ class VulnerabilityServiceChoice(str, enum.Enum):
 
     Osv = "osv"
     Pypi = "pypi"
-
-    def to_service(self, timeout: int, cache_dir: Path | None) -> VulnerabilityService:
-        if self is VulnerabilityServiceChoice.Osv:
-            return OsvService(cache_dir, timeout)
-        elif self is VulnerabilityServiceChoice.Pypi:
-            return PyPIService(cache_dir, timeout)
-        else:
-            assert_never(self)  # pragma: no cover
+    Esms = "esms"
 
     def __str__(self) -> str:
         return self.value
@@ -134,6 +135,30 @@ class VulnerabilityDescriptionChoice(str, enum.Enum):
         return self.value
 
 
+@enum.unique
+class VulnerabilityAliasChoice(str, enum.Enum):
+    """
+    Whether or not vulnerability aliases should be added to the `pip-audit` output.
+    """
+
+    On = "on"
+    Off = "off"
+    Auto = "auto"
+
+    def to_bool(self, format_: OutputFormatChoice) -> bool:
+        if self is VulnerabilityAliasChoice.On:
+            return True
+        elif self is VulnerabilityAliasChoice.Off:
+            return False
+        elif self is VulnerabilityAliasChoice.Auto:
+            return bool(format_ is OutputFormatChoice.Json)
+        else:
+            assert_never(self)  # pragma: no cover
+
+    def __str__(self) -> str:
+        return self.value
+
+
 @enum.unique
 class ProgressSpinnerChoice(str, enum.Enum):
     """
@@ -184,21 +209,30 @@ def _parser() -> argparse.ArgumentParser:  # pragma: no cover
     dep_source_args.add_argument(
         "-r",
         "--requirement",
-        type=argparse.FileType("r"),
+        type=Path,
         metavar="REQUIREMENT",
         action="append",
         dest="requirements",
         help="audit the given requirements file; this option can be used multiple times",
     )
     dep_source_args.add_argument(
-        "project_path", type=Path, nargs="?", help="audit a local Python project at the given path"
+        "project_path",
+        type=Path,
+        nargs="?",
+        help="audit a local Python project at the given path",
+    )
+    parser.add_argument(
+        "--locked",
+        action="store_true",
+        help="audit lock files from the local Python project. This "
+        "flag only applies to auditing from project paths",
     )
     parser.add_argument(
         "-f",
         "--format",
         type=OutputFormatChoice,
         choices=OutputFormatChoice,
-        default=OutputFormatChoice.Columns,
+        default=os.environ.get("PIP_AUDIT_FORMAT", OutputFormatChoice.Columns),
         metavar="FORMAT",
         help=_enum_help("the format to emit audit results in", OutputFormatChoice),
     )
@@ -207,12 +241,21 @@ def _parser() -> argparse.ArgumentParser:  # pragma: no cover
         "--vulnerability-service",
         type=VulnerabilityServiceChoice,
         choices=VulnerabilityServiceChoice,
-        default=VulnerabilityServiceChoice.Pypi,
+        default=os.environ.get("PIP_AUDIT_VULNERABILITY_SERVICE", VulnerabilityServiceChoice.Pypi),
         metavar="SERVICE",
         help=_enum_help(
-            "the vulnerability service to audit dependencies against", VulnerabilityServiceChoice
+            "the vulnerability service to audit dependencies against",
+            VulnerabilityServiceChoice,
         ),
     )
+    parser.add_argument(
+        "--osv-url",
+        type=str,
+        metavar="OSV_URL",
+        dest="osv_url",
+        default=os.environ.get("PIP_AUDIT_OSV_URL", OsvService.DEFAULT_OSV_URL),
+        help="URL to use for the OSV API instead of the default",
+    )
     parser.add_argument(
         "-d",
         "--dry-run",
@@ -232,11 +275,22 @@ def _parser() -> argparse.ArgumentParser:  # pragma: no cover
         choices=VulnerabilityDescriptionChoice,
         nargs="?",
         const=VulnerabilityDescriptionChoice.On,
-        default=VulnerabilityDescriptionChoice.Auto,
+        default=os.environ.get("PIP_AUDIT_DESC", VulnerabilityDescriptionChoice.Auto),
         help="include a description for each vulnerability; "
         "`auto` defaults to `on` for the `json` format. This flag has no "
         "effect on the `cyclonedx-json` or `cyclonedx-xml` formats.",
     )
+    parser.add_argument(
+        "--aliases",
+        type=VulnerabilityAliasChoice,
+        choices=VulnerabilityAliasChoice,
+        nargs="?",
+        const=VulnerabilityAliasChoice.On,
+        default=VulnerabilityAliasChoice.Auto,
+        help="includes alias IDs for each vulnerability; "
+        "`auto` defaults to `on` for the `json` format. This flag has no "
+        "effect on the `cyclonedx-json` or `cyclonedx-xml` formats.",
+    )
     parser.add_argument(
         "--cache-dir",
         type=Path,
@@ -246,11 +300,14 @@ def _parser() -> argparse.ArgumentParser:  # pragma: no cover
         "--progress-spinner",
         type=ProgressSpinnerChoice,
         choices=ProgressSpinnerChoice,
-        default=ProgressSpinnerChoice.On,
+        default=os.environ.get("PIP_AUDIT_PROGRESS_SPINNER", ProgressSpinnerChoice.On),
         help="display a progress spinner",
     )
     parser.add_argument(
-        "--timeout", type=int, default=15, help="set the socket timeout"  # Match the `pip` default
+        "--timeout",
+        type=int,
+        default=15,
+        help="set the socket timeout",  # Match the `pip` default
     )
     dep_source_args.add_argument(
         "--path",
@@ -313,7 +370,7 @@ def _parser() -> argparse.ArgumentParser:  # pragma: no cover
         type=Path,
         metavar="FILE",
         help="output results to the given file",
-        default="stdout",
+        default=os.environ.get("PIP_AUDIT_OUTPUT", "stdout"),
     )
     parser.add_argument(
         "--ignore-vuln",
@@ -327,6 +384,13 @@ def _parser() -> argparse.ArgumentParser:  # pragma: no cover
             "this option can be used multiple times"
         ),
     )
+    parser.add_argument(
+        "--disable-pip",
+        action="store_true",
+        help="don't use `pip` for dependency resolution; "
+        "this can only be used with hashed requirements files or if the `--no-deps` flag has been "
+        "provided",
+    )
     return parser
 
 
@@ -345,13 +409,28 @@ def _parse_args(parser: argparse.ArgumentParser) -> argparse.Namespace:  # pragm
 
 
 def _dep_source_from_project_path(
-    project_path: Path, index_url: str, extra_index_urls: list[str], state: AuditState
+    project_path: Path, index_url: str, extra_index_urls: list[str], locked: bool, state: AuditState
 ) -> DependencySource:  # pragma: no cover
+    # If the user has passed `--locked`, we check for `pylock.*.toml` files.
+    if locked:
+        all_pylocks = list(project_path.glob("pylock.*.toml"))
+        generic_pylock = project_path / "pylock.toml"
+        if generic_pylock.is_file():
+            all_pylocks.append(generic_pylock)
+
+        if not all_pylocks:
+            _fatal(f"no lockfiles found in {project_path}")
+
+        return PyLockSource(all_pylocks)
+
     # Check for a `pyproject.toml`
     pyproject_path = project_path / "pyproject.toml"
     if pyproject_path.is_file():
         return PyProjectSource(
-            pyproject_path, index_url=index_url, extra_index_urls=extra_index_urls, state=state
+            pyproject_path,
+            index_url=index_url,
+            extra_index_urls=extra_index_urls,
+            state=state,
         )
 
     # TODO: Checks for setup.py and other project files will go here.
@@ -366,9 +445,24 @@ def audit() -> None:  # pragma: no cover
     parser = _parser()
     args = _parse_args(parser)
 
-    service = args.vulnerability_service.to_service(args.timeout, args.cache_dir)
+    service: VulnerabilityService
+    if args.vulnerability_service is VulnerabilityServiceChoice.Osv:
+        service = OsvService(cache_dir=args.cache_dir, timeout=args.timeout, osv_url=args.osv_url)
+    elif args.vulnerability_service is VulnerabilityServiceChoice.Pypi:
+        service = PyPIService(cache_dir=args.cache_dir, timeout=args.timeout)
+    elif args.vulnerability_service is VulnerabilityServiceChoice.Esms:
+        service = EcosystemsService(cache_dir=args.cache_dir, timeout=args.timeout)
+    else:
+        assert_never(args.vulnerability_service)  # pragma: no cover
+
     output_desc = args.desc.to_bool(args.format)
-    formatter = args.format.to_format(output_desc)
+    output_aliases = args.aliases.to_bool(args.format)
+    formatter = args.format.to_format(output_desc, output_aliases)
+
+    # Check for flags that are only valid with project paths
+    if args.project_path is None:
+        if args.locked:
+            parser.error("The --locked flag can only be used with a project path")
 
     # Check for flags that are only valid with requirements files
     if args.requirements is None:
@@ -380,6 +474,8 @@ def audit() -> None:  # pragma: no cover
             parser.error("The --extra-index-url flag can only be used with --requirement (-r)")
         elif args.no_deps:
             parser.error("The --no-deps flag can only be used with --requirement (-r)")
+        elif args.disable_pip:
+            parser.error("The --disable-pip flag can only be used with --requirement (-r)")
 
     # Nudge users to consider alternate workflows.
     if args.require_hashes and args.no_deps:
@@ -409,11 +505,15 @@ def audit() -> None:  # pragma: no cover
 
         source: DependencySource
         if args.requirements is not None:
-            req_files: list[Path] = [Path(req.name) for req in args.requirements]
+            for req in args.requirements:
+                if not req.exists():
+                    _fatal(f"invalid requirements input: {req}")
+
             source = RequirementSource(
-                req_files,
+                args.requirements,
                 require_hashes=args.require_hashes,
                 no_deps=args.no_deps,
+                disable_pip=args.disable_pip,
                 skip_editable=args.skip_editable,
                 index_url=args.index_url,
                 extra_index_urls=args.extra_index_urls,
@@ -428,11 +528,15 @@ def audit() -> None:  # pragma: no cover
                 args.project_path,
                 args.index_url,
                 args.extra_index_urls,
+                args.locked,
                 state,
             )
         else:
             source = PipSource(
-                local=args.local, paths=args.paths, skip_editable=args.skip_editable, state=state
+                local=args.local,
+                paths=args.paths,
+                skip_editable=args.skip_editable,
+                state=state,
             )
 
         # `--dry-run` only affects the auditor if `--fix` is also not supplied,
@@ -440,7 +544,7 @@ def audit() -> None:  # pragma: no cover
         # wants to dry-run the "fix" step instead of the "audit" step
         auditor = Auditor(service, options=AuditOptions(dry_run=args.dry_run and not args.fix))
 
-        result = {}
+        result: dict[Dependency, list[VulnerabilityResult]] = {}
         pkg_count = 0
         vuln_count = 0
         skip_count = 0
@@ -494,9 +598,7 @@ def audit() -> None:  # pragma: no cover
                         )
                     else:
                         fix = cast(ResolvedFixVersion, fix)
-                        logger.info(
-                            f"Dry run: would have upgraded {fix.dep.name} to " f"{fix.version}"
-                        )
+                        logger.info(f"Dry run: would have upgraded {fix.dep.name} to {fix.version}")
                     continue
 
                 if not fix.is_skipped():
@@ -512,11 +614,15 @@ def audit() -> None:  # pragma: no cover
                 fixes.append(fix)
 
     if vuln_count > 0:
+        if vuln_ignore_count:
+            ignored = f", ignored {vuln_ignore_count}"
+        else:
+            ignored = ""
+
         summary_msg = (
             f"Found {vuln_count} known "
             f"{'vulnerability' if vuln_count == 1 else 'vulnerabilities'}"
-            f"{(vuln_ignore_count and ', ignored %d ' % vuln_ignore_count) or ' '}"
-            f"in {pkg_count} {'package' if pkg_count == 1 else 'packages'}"
+            f"{ignored} in {pkg_count} {'package' if pkg_count == 1 else 'packages'}"
         )
         if args.fix:
             summary_msg += (
