@@ -7,13 +7,17 @@ import logging
 import os
 import subprocess
 import sys
+from collections.abc import Iterator, Sequence
 from pathlib import Path
-from typing import Iterator, Sequence
 
 import pip_api
 from packaging.version import InvalidVersion, Version
 
-from pip_audit._dependency_source import DependencyFixError, DependencySource, DependencySourceError
+from pip_audit._dependency_source import (
+    DependencyFixError,
+    DependencySource,
+    DependencySourceError,
+)
 from pip_audit._fix import ResolvedFixVersion
 from pip_audit._service import Dependency, ResolvedDependency, SkippedDependency
 from pip_audit._state import AuditState
@@ -108,7 +112,7 @@ class PipSource(DependencySource):
         # The `pip list` call that underlies `pip_api` could fail for myriad reasons.
         # We collect them all into a single well-defined error.
         try:
-            for (_, dist) in pip_api.installed_distributions(
+            for _, dist in pip_api.installed_distributions(
                 local=self._local, paths=list(self._paths)
             ).items():
                 dep: Dependency
@@ -147,7 +151,10 @@ class PipSource(DependencySource):
         ]
         try:
             subprocess.run(
-                fix_cmd, check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL
+                fix_cmd,
+                check=True,
+                stdout=subprocess.DEVNULL,
+                stderr=subprocess.DEVNULL,
             )
         except subprocess.CalledProcessError as cpe:
             raise PipFixError(
