@@ -2,6 +2,7 @@
 Functionality for using the [PyPI](https://warehouse.pypa.io/api-reference/json.html)
 API as a `VulnerabilityService`.
 """
+
 from __future__ import annotations
 
 import logging
@@ -31,7 +32,9 @@ class PyPIService(VulnerabilityService):
     package vulnerability information.
     """
 
-    def __init__(self, cache_dir: Path | None = None, timeout: int | None = None) -> None:
+    def __init__(
+        self, cache_dir: Path | None = None, timeout: int | None = None, **kwargs: dict
+    ) -> None:
         """
         Create a new `PyPIService`.
 
@@ -84,24 +87,6 @@ class PyPIService(VulnerabilityService):
 
         response_json = response.json()
         results: list[VulnerabilityResult] = []
-
-        # If the dependency has a hash explicitly listed, check it against the PyPI data
-        if spec.hashes:
-            for hash_type, hash_values in spec.hashes.items():
-                for hash_value in hash_values:
-                    found = False
-                    for dist in response_json["urls"]:
-                        digests = dist["digests"]
-                        pypi_hash = digests.get(hash_type)
-                        if pypi_hash is not None and pypi_hash == hash_value:
-                            found = True
-                            break
-                    if not found:
-                        raise ServiceError(
-                            f"Mismatched hash for {spec.canonical_name} ({spec.version}): listed "
-                            f"{hash_value} of type {hash_type} could not be found in PyPI releases"
-                        )
-
         vulns = response_json.get("vulnerabilities")
 
         # No `vulnerabilities` key means that there are no vulnerabilities for any version
@@ -121,7 +106,7 @@ class PyPIService(VulnerabilityService):
             try:
                 fix_versions = [Version(fixed_in) for fixed_in in v["fixed_in"]]
             except InvalidVersion as iv:
-                raise ServiceError(f'Received malformed version from PyPI: {v["fixed_in"]}') from iv
+                raise ServiceError(f"Received malformed version from PyPI: {v['fixed_in']}") from iv
 
             # The ranges aren't guaranteed to come in chronological order
             fix_versions.sort()
@@ -140,11 +125,10 @@ class PyPIService(VulnerabilityService):
             description = description.replace("\n", " ")
 
             results.append(
-                VulnerabilityResult(
-                    id=id,
+                VulnerabilityResult.create(
+                    ids=[id, *v["aliases"]],
                     description=description,
                     fix_versions=fix_versions,
-                    aliases=set(v["aliases"]),
                     published=self._parse_rfc3339(v.get("published")),
                 )
             )
